from distutils.core import setup

setup(name='mattys_mocks',
      version='0.1.1',
      description='Yet another mocking library',
      author='Matt ODonnell',
      author_email='matty@mavenrd.com',
      url='https://github.com/mattyo/mattysmocks',
      packages=['mattys_mocks', 'mattys_mocks.tests', 'mattys_mocks.tests.mock_these'],
     )
