class DumbClass():
    a_attribute = 'test attribute'
    b_attribute = 'test b attribute'

    def a_method(self):
        pass

    def b_method(self):
        pass

    @property
    def a_property(self):
        pass

    @classmethod
    def class_method(self):
        pass
    @staticmethod
    def static_method():
        pass
